using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;

namespace Cotyounoyume{
    public struct Offset{
        public float Up;
        public float Forward;
        public float Right;

        public Offset(float Up, float Forward, float Right){
            this.Up = Up;
            this.Forward = Forward;
            this.Right = Right;
        }
    };

	public class BodyPartLocator
	{
        public EBATCore core;
        public Atom atom;
        public Dictionary<string, Offset> BodyPartPositionOffset;
        public Dictionary<string, Quaternion> BodyPartRotation;
        public List<string> BodyPartNames = new List<string>()
        {
            "rNipple", "lNipple", "LabiaTrigger", "Anus", "lShin", "rShin",
        };
        public float fluctuation = 0.01f;
        public Dictionary<string, Rigidbody> RigidbodyDic = new Dictionary<string, Rigidbody>();

        public Dictionary<string, FreeControllerV3> ControllerDic = new Dictionary<string, FreeControllerV3>();

        public BodyPartLocator(EBATCore core)
        {
            this.core = core;
            this.atom = core.atom;
            BodyPartPositionOffset = new Dictionary<string, Offset>(){
                {"LabiaTrigger",     new Offset(-0.05f,  0.00f, 0f)},
                {"Anus",             new Offset( 0.00f, -0.05f, 0f)},
                {"rNipple",          new Offset( 0.02f,  0.05f, -0.02f)},
                {"lNipple",          new Offset( 0.02f,  0.05f,  0.02f)},
                {"lShin",            new Offset( 0.05f,  0.05f, 0f)},
                {"rShin",            new Offset( 0.05f,  0.05f, 0f)},
                {"HandL_from_chest", new Offset(-0.20f,  1.0f, -0.20f)},
                {"HandR_from_chest", new Offset(-0.20f,  1.0f,  0.20f)},
                {"HandLR",           new Offset(-0.01f, -0.01f, 0f)},
                {"HandLR1",          new Offset(-0.01f, -0.01f, 0f)},
                {"TwinDildo",        new Offset(-0.02f, 0.00f, 0f)},
                {"TwinDildoV",       new Offset(0.3f, 0.3f, -0.6f)},
                {"WMNZ",             new Offset(-0.02f, 0.00f, 0f)},
                {"Tentacle1",        new Offset(0.08f, 0f, 0f)},
                {"HandL",            new Offset(0.0f, 0.0f, 0f)},
                {"HandL2",           new Offset( 0.00f, -0.05f, 0f)},
                {"HandR",            new Offset(-0.02f, 0.0f, 0.0f)},
                {"Mouth1L",          new Offset(0f, -0.005f, 0f)},
                {"Mouth1R",          new Offset(0f, -0.01f, 0.005f)},
                {"Lip",              new Offset(0f, 0f, 0f)},
            };
            BodyPartRotation = new Dictionary<string, Quaternion>();
            ControllerList.ForEach((name) => ControllerDic.Add(name, this.atom.GetStorableByID(name) as FreeControllerV3));
            RigidbodyList.ForEach((name) => RigidbodyDic.Add(name, Utility.FindRigidBody(this.atom, name)));
        }

        public FreeControllerV3 GetController(string name)
        {
            if(!ControllerDic.ContainsKey(name)) return null;
            return ControllerDic[name];
        }

        public FreeControllerV3 GetRelativeController(string name)
        {
            string altName = "";
            if(name == "lHandControl") altName = "lElbowControl";
            else if(name == "rHandControl") altName = "rElbowControl";
            else if(name == "rKneeControl") altName = "lKneeControl";
            else if(name == "lKneeControl") altName = "rKneeControl";

            if(!ControllerDic.ContainsKey(altName)) return null;
            return ControllerDic[altName];
        }

        public Rigidbody GetRigidbody(string name)
        {
            if(!RigidbodyDic.ContainsKey(name)) return null;
            return RigidbodyDic[name];
        }

        public Offset GetOffset(string name)
        {
            if(!BodyPartPositionOffset.ContainsKey(name)) new Vector3(0f, 0f, 0f);
            return BodyPartPositionOffset[name];
        }

        public Vector3 GetPosition(string name, bool isBothHandsOn = false)
        {
            try{
                var bothHandsOffset = isBothHandsOn ? 0.15f : 0.0f;
                Offset offset = GetOffset(name);
                // Vector3 v3 = Utility.GetPositionFromTargetBodynameWithOffset(atom, name, offset.Up, offset.Forward + bothHandsOffset, offset.Right, fluctuation);
                return Utility.GetPositionFromTargetBodynameWithOffset(atom, name, offset.Up, offset.Forward + bothHandsOffset, offset.Right, fluctuation);
            }catch(Exception e){
                Utility.LoggerE(e);
                return new Vector3();
            }
        }

        public void OnDestroy()
        {
            BodyPartPositionOffset.Clear();
            BodyPartRotation.Clear();
        }

		public void SetControllersAndRigidbodies(){
				
		}

        public List<string> ControllerList = new List<string>(){
            "control",
            "eyeTargetControl",
            "headControl",
            "neckControl",
            "chestControl",
            "rShoulderControl",
            "lShoulderControl",
            "rArmControl",
            "lArmControl",
            "rElbowControl",
            "lElbowControl",
            "rHandControl",
            "lHandControl",
            "abdomenControl",
            "abdomen2Control",
            "hipControl",
            "pelvisControl",
            "rThighControl",
            "lThighControl",
            "rKneeControl",
            "lKneeControl",
            "rFootControl",
            "lFootControl",
        };

        public List<string> RigidbodyList = new List<string>(){
            "head",
            "neck",
            "chest",
            "lCollar",
            "rCollar",
            "lShldr",
            "rShldr",
            "lPectoral",
            "rPectoral",
            "lNipple",
            "rNipple",
            "lForeArm",
            "rForeArm",
            "lHand",
            "rHand",
            "abdomen",
            "abdomen2",
            "hip",
            "pelvis",
            "lThigh",
            "rThigh",
            "lShin",
            "rShin",
            "lFoot",
            "rFoot",
            "LabiaTrigger",
        };
    }
}